@@ -1,3 +1,5 @@
+//! The compiler module that transforms [CaoIr](CaoIr) into bytecode.
+//!
 mod card;
 mod compilation_error;
 mod compile_options;
@@ -10,11 +12,11 @@ mod tests;
 
 use crate::{
     bytecode::{encode_str, write_to_vec},
-    collections::key_map::{Key, KeyMap},
+    collections::key_map::{Handle, KeyMap},
+    instruction::instruction_span,
     program::{CaoProgram, Label},
-    Instruction, VarName,
+    Instruction, NodeId, VarName, VariableId,
 };
-use crate::{NodeId, VariableId};
 use std::fmt::Debug;
 use std::mem;
 use std::{cell::RefCell, convert::TryFrom};
@@ -41,20 +43,18 @@ pub struct Compiler<'a> {
     pub(crate) program: CaoProgram,
     next_var: RefCell<VariableId>,
 
-    /// maps lanes to their pre-hash-map keys
+    /// maps lanes to their metadata
     jump_table: KeyMap<LaneMeta>,
 
     locals: Box<arrayvec::ArrayVec<Local<'a>, 255>>,
     scope_depth: i32,
     current_card: i32,
     current_lane: LaneNode,
-
-    _m: std::marker::PhantomData<&'a ()>,
 }
 
 #[derive(Debug, Clone, Copy)]
 struct LaneMeta {
-    pub hash_key: Key,
+    pub hash_key: Handle,
     /// number of arguments
     pub arity: u32,
 }
@@ -68,7 +68,7 @@ pub struct Local<'a> {
 }
 
 pub fn compile(
-    compilation_unit: CaoIr,
+    compilation_unit: &CaoIr,
     compile_options: impl Into<Option<CompileOptions>>,
 ) -> CompilationResult<CaoProgram> {
     let mut compiler = Compiler::new();
@@ -92,13 +92,12 @@ impl<'a> Compiler<'a> {
             scope_depth: 0,
             current_lane: LaneNode::LaneId(0),
             current_card: -1,
-            _m: Default::default(),
         }
     }
 
     pub fn compile(
         &mut self,
-        compilation_unit: CaoIr,
+        compilation_unit: &CaoIr,
         compile_options: impl Into<Option<CompileOptions>>,
     ) -> CompilationResult<CaoProgram> {
         self.options = compile_options.into().unwrap_or_default();
@@ -106,7 +105,7 @@ impl<'a> Compiler<'a> {
         self._compile(compilation_unit)
     }
 
-    fn _compile(&mut self, mut compilation_unit: CaoIr) -> CompilationResult<CaoProgram> {
+    fn _compile(&mut self, compilation_unit: &CaoIr) -> CompilationResult<CaoProgram> {
         if compilation_unit.lanes.is_empty() {
             return Err(CompilationError::with_loc(
                 CompilationErrorPayload::EmptyProgram,
@@ -117,7 +116,7 @@ impl<'a> Compiler<'a> {
         // initialize
         self.program = CaoProgram::default();
         self.next_var = RefCell::new(VariableId(0));
-        self.compile_stage_1(&mut compilation_unit)?;
+        self.compile_stage_1(compilation_unit)?;
         self.compile_stage_2(compilation_unit)?;
 
         Ok(mem::take(&mut self.program))
@@ -129,7 +128,7 @@ impl<'a> Compiler<'a> {
 
     /// build the jump table and consume the lane names
     /// also reserve memory for the program labels
-    fn compile_stage_1(&mut self, compilation_unit: &mut CaoIr) -> CompilationResult<()> {
+    fn compile_stage_1(&mut self, compilation_unit: &CaoIr) -> CompilationResult<()> {
         // check if len fits in 16 bits
         let _: u16 = match compilation_unit.lanes.len().try_into() {
             Ok(i) => i,
@@ -138,14 +137,14 @@ impl<'a> Compiler<'a> {
 
         let mut num_cards = 0usize;
         self.current_card = -1;
-        for (i, n) in compilation_unit.lanes.iter_mut().enumerate() {
+        for (i, n) in compilation_unit.lanes.iter().enumerate() {
             self.current_lane = LaneNode::LaneId(i);
 
-            let indexkey = Key::from_u32(i as u32);
+            let indexkey = Handle::from_i64(i as i64);
             assert!(!self.jump_table.contains(indexkey));
             num_cards += n.cards.len();
 
-            let nodekey = Key::from_u32(
+            let nodekey = Handle::from_u32(
                 NodeId {
                     // we know that i fits in 16 bits from the check above
                     lane: i as u16,
@@ -159,12 +158,10 @@ impl<'a> Compiler<'a> {
                 arity: n.arguments.len() as u32,
             };
             self.jump_table.insert(indexkey, metadata).unwrap();
-            if let Some(ref mut name) = n.name.as_mut() {
-                let namekey = Key::from_str(name.as_str()).expect("Failed to hash lane name");
+            if let Some(name) = n.name.as_ref() {
+                let namekey = Handle::from_str(name.as_str()).expect("Failed to hash lane name");
                 if self.jump_table.contains(namekey) {
-                    return Err(
-                        self.error(CompilationErrorPayload::DuplicateName(std::mem::take(name)))
-                    );
+                    return Err(self.error(CompilationErrorPayload::DuplicateName(name.clone())));
                 }
                 self.jump_table.insert(namekey, metadata).unwrap();
             }
@@ -175,8 +172,8 @@ impl<'a> Compiler<'a> {
     }
 
     /// consume lane cards and build the bytecode
-    fn compile_stage_2(&mut self, compilation_unit: CaoIr) -> CompilationResult<()> {
-        let mut lanes = compilation_unit.lanes.into_iter().enumerate();
+    fn compile_stage_2(&mut self, compilation_unit: &CaoIr) -> CompilationResult<()> {
+        let mut lanes = compilation_unit.lanes.iter().enumerate();
 
         if let Some((il, main_lane)) = lanes.next() {
             let len: u16 = match main_lane.cards.len().try_into() {
@@ -191,7 +188,7 @@ impl<'a> Compiler<'a> {
             };
             self.scope_end();
             // insert explicit exit after the first lane
-            self.process_card(nodeid, Card::Abort)?;
+            self.process_card(nodeid, &Card::Abort)?;
         }
 
         for (il, lane) in lanes {
@@ -200,7 +197,7 @@ impl<'a> Compiler<'a> {
                     lane: il as u16,
                     pos: 0,
                 };
-                let nodeid_hash = Key::from_u32(nodeid.into());
+                let nodeid_hash = Handle::from_u32(nodeid.into());
                 let handle = u32::try_from(self.program.bytecode.len())
                     .expect("bytecode length to fit into 32 bits");
                 self.program
@@ -260,7 +257,7 @@ impl<'a> Compiler<'a> {
         il: usize,
         Lane {
             cards, arguments, ..
-        }: Lane,
+        }: &Lane,
         // cards: Vec<Card>,
         instruction_offset: i32,
     ) -> CompilationResult<()> {
@@ -278,7 +275,7 @@ impl<'a> Compiler<'a> {
                 self.error(CompilationErrorPayload::BadVariableName(param.to_string()))
             })?)?;
         }
-        for (ic, card) in cards.into_iter().enumerate() {
+        for (ic, card) in cards.iter().enumerate() {
             self.current_card = ic as i32;
             let nodeid = NodeId {
                 lane: il as u16,
@@ -312,29 +309,13 @@ impl<'a> Compiler<'a> {
     }
 
     fn encode_jump(&mut self, nodeid: NodeId, lane: &LaneNode) -> CompilationResult<()> {
-        let to = match lane {
-            LaneNode::LaneName(lane) => self
-                .jump_table
-                .get(Key::from_str(lane).expect("Failed to hash jump target name"))
-                .ok_or_else(|| {
-                    self.error(CompilationErrorPayload::InvalidJump {
-                        src: nodeid,
-                        dst: lane.to_string(),
-                        msg: None,
-                    })
-                })?,
-            LaneNode::LaneId(id) => {
-                self.jump_table
-                    .get(Key::from_u32(*id as u32))
-                    .ok_or_else(|| {
-                        self.error(CompilationErrorPayload::InvalidJump {
-                            src: nodeid,
-                            dst: format!("Lane id {}", id),
-                            msg: None,
-                        })
-                    })?
-            }
-        };
+        let to = self.jump_table.get(Handle::from(lane)).ok_or_else(|| {
+            self.error(CompilationErrorPayload::InvalidJump {
+                src: nodeid,
+                dst: lane.clone(),
+                msg: None,
+            })
+        })?;
         write_to_vec(to.hash_key, &mut self.program.bytecode);
         write_to_vec(to.arity, &mut self.program.bytecode);
         Ok(())
@@ -357,10 +338,10 @@ impl<'a> Compiler<'a> {
         Ok(-1)
     }
 
-    fn process_card(&mut self, nodeid: NodeId, card: Card) -> CompilationResult<()> {
+    fn process_card(&mut self, nodeid: NodeId, card: &Card) -> CompilationResult<()> {
         let handle = u32::try_from(self.program.bytecode.len())
             .expect("bytecode length to fit into 32 bits");
-        let nodeid_hash = Key::from_u32(nodeid.into());
+        let nodeid_hash = Handle::from_u32(nodeid.into());
         self.program
             .labels
             .0
@@ -372,67 +353,47 @@ impl<'a> Compiler<'a> {
             self.program.bytecode.push(instr as u8);
         }
         match card {
+            Card::ForEach { variable, lane } => {
+                let target_lane = Handle::from(lane);
+                let arity = self.jump_table[target_lane].arity;
+                if arity != 2 {
+                    return Err(self.error(CompilationErrorPayload::InvalidJump {
+                        src: nodeid,
+                        dst: lane.clone(),
+                        msg: Some(
+                            "ForEach lanes need to have 2 parameters: `key` and `table` where the `key` is the current key, and `table` is the table that's being iterated over"
+                            .to_string()
+                        )
+                    }));
+                }
+                self.read_var_card(variable)?;
+                self.program.bytecode.push(Instruction::BeginForEach as u8);
+                let block_begin = self.program.bytecode.len() as i32;
+                self.program.bytecode.push(Instruction::ForEach as u8);
+                self.encode_jump(nodeid, lane)?;
+                // return to the repeat instruction
+                self.program.bytecode.push(Instruction::GotoIfTrue as u8);
+                write_to_vec(block_begin, &mut self.program.bytecode);
+            }
             // TODO: blocked by lane ABI
             Card::While(_) => {
                 return Err(self.error(CompilationErrorPayload::Unimplemented("While cards")))
             }
             Card::Repeat(repeat) => {
-                // Init, add 1
-                self.program.bytecode.push(Instruction::ScalarInt as u8);
-                write_to_vec(1i64, &mut self.program.bytecode);
-                self.program.bytecode.push(Instruction::Add as u8);
-                // Condition
-                let cond_block_begin = self.program.bytecode.len() as i32;
-                self.program.bytecode.push(Instruction::ScalarInt as u8);
-                write_to_vec(1i64, &mut self.program.bytecode);
-                self.program.bytecode.push(Instruction::Sub as u8);
-                self.program.bytecode.push(Instruction::CopyLast as u8);
-                self.program.bytecode.push(Instruction::GotoIfFalse as u8);
-                let execute_block_len =
-                    instruction_span(Instruction::CallLane) + instruction_span(Instruction::Goto);
-                write_to_vec(
-                    self.program.bytecode.len() as i32 + 4 + execute_block_len,
-                    &mut self.program.bytecode,
-                );
-                // Execute
-                self.program.bytecode.push(Instruction::CallLane as u8);
-                self.encode_jump(nodeid, &repeat)?;
-                self.program.bytecode.push(Instruction::Goto as u8);
-                write_to_vec(cond_block_begin, &mut self.program.bytecode);
+                self.program.bytecode.push(Instruction::BeginRepeat as u8);
+                let block_begin = self.program.bytecode.len() as i32;
+                self.program.bytecode.push(Instruction::Repeat as u8);
+                self.encode_jump(nodeid, repeat)?;
+                // return to the repeat instruction
+                self.program.bytecode.push(Instruction::GotoIfTrue as u8);
+                write_to_vec(block_begin, &mut self.program.bytecode);
             }
             Card::ReadVar(variable) => {
-                let scope = self.resolve_var(variable.0.as_str())?;
-                if scope < 0 {
-                    // global
-                    let mut next_var = self.next_var.borrow_mut();
-                    let varhash = Key::from_bytes(variable.0.as_bytes());
-                    let id = self
-                        .program
-                        .variables
-                        .ids
-                        .entry(varhash)
-                        .or_insert_with(move || {
-                            let id = *next_var;
-                            *next_var = VariableId(id.0 + 1);
-                            id
-                        });
-                    self.program
-                        .variables
-                        .names
-                        .entry(*id)
-                        .or_insert_with(|| variable.0);
-                    self.program.bytecode.push(Instruction::ReadGlobalVar as u8);
-                    write_to_vec(*id, &mut self.program.bytecode);
-                } else {
-                    //local
-                    self.program.bytecode.push(Instruction::ReadLocalVar as u8);
-                    let index = scope as u32;
-                    write_to_vec(index, &mut self.program.bytecode);
-                }
+                self.read_var_card(variable)?;
             }
             Card::SetVar(var) => {
                 let index = self.locals.len() as u32;
-                self.add_local(var.0)?;
+                self.add_local(*var.0)?;
                 self.program.bytecode.push(Instruction::SetLocalVar as u8);
                 write_to_vec(index, &mut self.program.bytecode);
             }
@@ -441,7 +402,7 @@ impl<'a> Compiler<'a> {
                 if variable.0.is_empty() {
                     return Err(self.error(CompilationErrorPayload::EmptyVariable));
                 }
-                let varhash = Key::from_bytes(variable.0.as_bytes());
+                let varhash = Handle::from_bytes(variable.0.as_bytes());
 
                 let id = self
                     .program
@@ -457,7 +418,7 @@ impl<'a> Compiler<'a> {
                     .variables
                     .names
                     .entry(*id)
-                    .or_insert_with(move || variable.0);
+                    .or_insert_with(move || *variable.0);
                 write_to_vec(*id, &mut self.program.bytecode);
             }
             Card::IfElse {
@@ -474,7 +435,7 @@ impl<'a> Compiler<'a> {
                 write_to_vec(pos, &mut self.program.bytecode);
                 // else
                 self.program.bytecode.push(Instruction::CallLane as u8);
-                self.encode_jump(nodeid, &else_lane)?;
+                self.encode_jump(nodeid, else_lane)?;
 
                 self.program.bytecode.push(Instruction::Goto as u8);
                 let pos = instruction_span(Instruction::CallLane)
@@ -483,23 +444,23 @@ impl<'a> Compiler<'a> {
                 write_to_vec(pos, &mut self.program.bytecode);
                 // then
                 self.program.bytecode.push(Instruction::CallLane as u8);
-                self.encode_jump(nodeid, &then_lane)?;
+                self.encode_jump(nodeid, then_lane)?;
             }
             Card::IfFalse(jmp) => {
                 // if the value is true we DON'T jump
-                self.conditional_jump(Instruction::GotoIfTrue, nodeid, &jmp)?;
+                self.conditional_jump(Instruction::GotoIfTrue, nodeid, jmp)?;
             }
             Card::IfTrue(jmp) => {
                 // if the value is false we DON'T jump
-                self.conditional_jump(Instruction::GotoIfFalse, nodeid, &jmp)?;
+                self.conditional_jump(Instruction::GotoIfFalse, nodeid, jmp)?;
             }
             Card::Jump(jmp) => {
-                self.encode_jump(nodeid, &jmp)?;
+                self.encode_jump(nodeid, jmp)?;
             }
             Card::StringLiteral(c) => self.push_str(c.0.as_str()),
             Card::CallNative(c) => {
                 let name = &c.0;
-                let key = Key::from_str(name.as_str()).unwrap();
+                let key = Handle::from_str(name.as_str()).unwrap();
                 write_to_vec(key, &mut self.program.bytecode);
             }
             Card::ScalarInt(s) => {
@@ -508,11 +469,6 @@ impl<'a> Compiler<'a> {
             Card::ScalarFloat(s) => {
                 write_to_vec(s.0, &mut self.program.bytecode);
             }
-            Card::GetProperty(VarNode(name)) | Card::SetProperty(VarNode(name)) => {
-                self.validate_var_name(name.as_str())?;
-                let handle = Key::from_str(name.as_str()).unwrap();
-                write_to_vec(handle, &mut self.program.bytecode);
-            }
             Card::ScalarNil
             | Card::Return
             | Card::And
@@ -532,11 +488,46 @@ impl<'a> Compiler<'a> {
             | Card::Mul
             | Card::Div
             | Card::CreateTable
+            | Card::Len
+            | Card::GetProperty
+            | Card::SetProperty
             | Card::ClearStack => { /* These cards translate to a single instruction */ }
         }
         Ok(())
     }
 
+    fn read_var_card(&mut self, variable: &VarNode) -> CompilationResult<()> {
+        let scope = self.resolve_var(variable.0.as_str())?;
+        if scope < 0 {
+            // global
+            let mut next_var = self.next_var.borrow_mut();
+            let varhash = Handle::from_bytes(variable.0.as_bytes());
+            let id = self
+                .program
+                .variables
+                .ids
+                .entry(varhash)
+                .or_insert_with(move || {
+                    let id = *next_var;
+                    *next_var = VariableId(id.0 + 1);
+                    id
+                });
+            self.program
+                .variables
+                .names
+                .entry(*id)
+                .or_insert_with(|| *variable.0);
+            self.program.bytecode.push(Instruction::ReadGlobalVar as u8);
+            write_to_vec(*id, &mut self.program.bytecode);
+        } else {
+            //local
+            self.program.bytecode.push(Instruction::ReadLocalVar as u8);
+            let index = scope as u32;
+            write_to_vec(index, &mut self.program.bytecode);
+        }
+        Ok(())
+    }
+
     fn validate_var_name(&self, name: &str) -> CompilationResult<()> {
         if name.is_empty() {
             return Err(self.error(CompilationErrorPayload::EmptyVariable));
@@ -544,44 +535,3 @@ impl<'a> Compiler<'a> {
         Ok(())
     }
 }
-
-/// return the number of bytes this instruction spans in the bytecode
-const fn instruction_span(instr: Instruction) -> i32 {
-    match instr {
-        Instruction::Add
-        | Instruction::Sub
-        | Instruction::Exit
-        | Instruction::Mul
-        | Instruction::Div
-        | Instruction::Call
-        | Instruction::Equals
-        | Instruction::NotEquals
-        | Instruction::Less
-        | Instruction::LessOrEq
-        | Instruction::Pop
-        | Instruction::Pass
-        | Instruction::ScalarNil
-        | Instruction::ClearStack
-        | Instruction::CopyLast
-        | Instruction::Return
-        | Instruction::SwapLast
-        | Instruction::And
-        | Instruction::Or
-        | Instruction::Xor
-        | Instruction::InitTable
-        | Instruction::Not => 1,
-        //
-        Instruction::ScalarInt | Instruction::ScalarFloat => 9,
-        Instruction::StringLiteral => 5,
-        //
-        Instruction::SetLocalVar
-        | Instruction::ReadLocalVar
-        | Instruction::SetProperty
-        | Instruction::GetProperty
-        | Instruction::SetGlobalVar
-        | Instruction::ReadGlobalVar => 5,
-        //
-        Instruction::Goto | Instruction::GotoIfTrue | Instruction::GotoIfFalse => 5,
-        Instruction::CallLane => 9,
-    }
-}
