@@ -1,6 +1,3 @@
-mod fibonacci;
-
-use std::convert::TryInto;
 use test_env_log::test;
 
 use cao_lang::{
@@ -25,12 +22,8 @@ fn test_string_w_utf8() {
     vm.run(&program).expect("run");
 
     let varid = program.variable_id("result").expect("varid");
-    let ptr = vm
-        .read_var(varid)
-        .expect("read var")
-        .try_into()
-        .expect("Expected a pointer");
-    let ress = unsafe { vm.get_str(ptr).expect("Failed to read string from vm") };
+    let ptr = vm.read_var(varid).expect("read var");
+    let ress = unsafe { ptr.as_str().expect("Failed to read string") };
 
     assert_eq!(test_str, ress);
 }
@@ -45,7 +38,7 @@ fn test_string_param() {
     }
 
     let fun = move |vm: &mut Vm<State>, arg: cao_lang::StrPointer| {
-        let vm_str = unsafe { vm.get_str(StrPointer(arg.0)).unwrap().to_string() };
+        let vm_str = unsafe { arg.get_str().unwrap().to_string() };
         vm.auxiliary_data.res = vm_str;
         Ok(())
     };
@@ -493,19 +486,19 @@ fn len_test_happy() {
         lanes: vec![Lane::default()
             .with_name("main")
             .with_card(Card::CreateTable)
-            .with_card(Card::SetVar(t))
+            .with_card(Card::SetVar(t.clone()))
             // first property
-            .with_card(Card::ReadVar(t))
+            .with_card(Card::ReadVar(t.clone()))
             .with_card(Card::StringLiteral(StringNode("asd".to_string())))
             .with_card(Card::ScalarInt(IntegerNode(42)))
             .with_card(Card::SetProperty)
             // same property as above
-            .with_card(Card::ReadVar(t))
+            .with_card(Card::ReadVar(t.clone()))
             .with_card(Card::StringLiteral(StringNode("asd".to_string())))
             .with_card(Card::ScalarInt(IntegerNode(42)))
             .with_card(Card::SetProperty)
             // new property
-            .with_card(Card::ReadVar(t))
+            .with_card(Card::ReadVar(t.clone()))
             .with_card(Card::StringLiteral(StringNode("basdasd".to_string())))
             .with_card(Card::ScalarInt(IntegerNode(42)))
             .with_card(Card::SetProperty)
@@ -524,78 +517,3 @@ fn len_test_happy() {
 
     assert_eq!(len, Value::Integer(2));
 }
-
-#[test]
-fn for_each_1() {
-    let cu: CaoIr = serde_yaml::from_str(
-        r#"
-lanes: 
-    - name: main
-      cards:
-        # --- init the example table ---
-        - ty: CreateTable
-        - ty: SetVar
-          val: t
-        - ty: ReadVar
-          val: t
-        - ty: StringLiteral
-          val: f1
-        - ty: ScalarInt
-          val: 1
-        - ty: SetProperty
-        - ty: ReadVar
-          val: t
-        - ty: StringLiteral
-          val: f2
-        - ty: ScalarInt
-          val: 2
-        - ty: SetProperty
-        - ty: ReadVar
-          val: t
-        - ty: StringLiteral
-          val: f3
-        - ty: ScalarInt
-          val: 3
-        - ty: SetProperty
-        - ty: ScalarInt
-          val: 0
-        - ty: SetGlobalVar
-          val: g_result
-        # --- init done ---
-        - ty: ForEach
-          val:
-            lane: 
-              LaneName: pog
-            variable: t
-
-
-    - name: pog
-      arguments:
-        - "key"
-        - "table"
-      cards:
-        - ty: ReadVar
-          val: table
-        - ty: ReadVar
-          val: key
-        - ty: GetProperty
-        - ty: ReadVar
-          val: g_result
-        - ty: Add
-        - ty: SetGlobalVar
-          val: g_result
-        "#,
-    )
-    .unwrap();
-
-    let program = compile(&cu, CompileOptions::new()).expect("compile");
-
-    let mut vm = Vm::new(()).unwrap();
-    vm.run(&program).expect("run");
-
-    let res = vm
-        .read_var_by_name("g_result", &program.variables)
-        .expect("Failed to read result variable");
-
-    assert_eq!(res, Value::Integer(6));
-}
